@@ -150,15 +150,20 @@ fn find_all_windows(
                     }
 
                     let executable = capture.get(6).unwrap().as_str();
-                    let version = format!("-{major}.{minor}-{pointer_width}");
-                    let output = Command::new(executable)
-                        .args(["-c", code])
-                        .output()
-                        .unwrap();
+                    let output = Command::new(executable).args(["-c", code]).output();
+                    let output = match output {
+                        Ok(output) => output,
+                        Err(err) => {
+                            eprintln!(
+                                "⚠️  Warning: failed to determine the path to python for `{executable}`: {err}"
+                            );
+                            continue;
+                        }
+                    };
                     let path = str::from_utf8(&output.stdout).unwrap().trim();
                     if !output.status.success() || path.trim().is_empty() {
                         eprintln!(
-                            "⚠️  Warning: couldn't determine the path to python for `py {version}`"
+                            "⚠️  Warning: couldn't determine the path to python for `{executable}`"
                         );
                         continue;
                     }
@@ -520,18 +525,15 @@ impl PythonInterpreter {
                     )
                 }
                 InterpreterKind::GraalPy => {
-                    // GraalPy suffers from pypa/packaging#614, where
-                    // packaging misdetects it as a 32-bit implementation,
-                    // so GraalPy adds the correct platform itself, e.g.
-                    // graalpy 3.10 23.1 => numpy-1.23.5-graalpy310-graalpy231_310_native_x86_64_linux-linux_i686.whl
+                    // GraalPy like PyPy uses its version as part of the ABI
+                    // graalpy 3.10 23.1 => numpy-1.23.5-graalpy310-graalpy231_310_native_manylinux2014_x86_64.whl
                     format!(
-                        "graalpy{major}{minor}-{abi_tag}_{arch}_{os}-{os}_i686",
+                        "graalpy{major}{minor}-{abi_tag}_{platform}",
                         major = self.major,
                         minor = self.minor,
                         abi_tag = calculate_abi_tag(&self.ext_suffix)
                             .expect("GraalPy's syconfig didn't define a valid `EXT_SUFFIX` ಠ_ಠ"),
-                        os = target.get_python_os(),
-                        arch = target.get_python_arch(),
+                        platform = platform,
                     )
                 }
             }
@@ -577,6 +579,7 @@ impl PythonInterpreter {
         bridge: &BridgeModel,
     ) -> Result<Option<PythonInterpreter>> {
         let output = Command::new(executable.as_ref())
+            .env("PYTHONNOUSERSITE", "1")
             .args(["-c", GET_INTERPRETER_METADATA])
             .output();
 
@@ -621,7 +624,8 @@ impl PythonInterpreter {
                             cmd.arg("/c")
                                 .arg("py")
                                 .arg(format!("-{}-{}", ver, target.pointer_width()))
-                                .arg(metadata_py.path());
+                                .arg(metadata_py.path())
+                                .env("PYTHONNOUSERSITE", "1");
                             let output = cmd.output();
                             match output {
                                 Ok(output) if output.status.success() => output,
